#! /usr/bin/python

#
# ShicksMessageList.py                                                             
#                                                                      
# Copyright (C) 2001,2002 Gerson Kurz (not@p-nand-q.com)
# BSD-Licensed. This is free software, enjoy!

import maildb, rotor, binascii, os

class MysqlMessageEntry:
    def __init__(self, data = None):
        if data is not None:
            self.msgid = data[0]
            self.userid = data[1]
            self.filename = data[2]
            self.msglen = data[3]
            self.deleted = data[4]
            self.created = data[5]
            self.msgtype = data[6]
        else:
            # this is a new message
            self.msgid = None

    def getId(self):
        return self.msgid

    def getUserId(self):
        return self.userid

    def setUserId(self,userid):
        self.userid = userid

    def getFilename(self):
        return self.filename

    def setFilename(self, filename):
        self.filename = filename

    def getMessageSize(self):
        return self.msglen

    def setMessageSize(self, msglen):
        self.msglen = msglen

    def getDeleted(self):
        return self.deleted

    def setDeleted(self, deleted):
        self.deleted = deleted       

    def getCreated(self):
        return self.created

    def setCreated(self, created):
        self.created = created       

    def getMessageType(self):
        return self.msgtype

    def setMessageType(self, msgtype):
        self.msgtype = msgtype
        
    def saveChanges(self):
        if self.msgid is None:
            # create a new message
            maildb.DbCommit("INSERT INTO messages (UID,BODY,MSGLEN,DELETED,CREATED,TYPE) VALUES (%d,'%s',%d,%d,NOW(),%d)" % (self.userid,maildb.EscapeSqlString(self.filename),self.msglen,self.deleted,self.msgtype))
        else:
            # only the DELETED flag can be changed. All other flags are fixed.
            maildb.DbCommit("UPDATE messages SET DELETED=%d WHERE ID=%d" % (self.deleted,self.msgid))
       
# ------------------ METHODS NEEDED FOR POP3Server.py / SMTPServer.py ------------------------------

def DeleteMessageBodies(idlist,delete_physically):
    "Given a list of IDs, delete all messages and associated files. Used for DELE-POP3 command. If delete_physically is true, will remove files also"

    idstringlist = ",".join(map(str,idlist))
    if delete_physically:
        for body in maildb.DbQuery("SELECT BODY FROM messages WHERE ID IN (%s)" % idstringlist):
            try:
                os.remove(body[0])
            except:
                pass
        maildb.DbCommit("DELETE FROM messages WHERE ID IN (%s)" % idstringlist)
    else:
        maildb.DbCommit("UPDATE messages SET DELETED=1 WHERE ID IN (%s)" % idstringlist)

def CreateEmptyMessage():
    return MysqlMessageEntry()

def GetUnreadMessagesForUserID(userid):
    "Given a user ID, return a list of unread messages for a given user id"
    table = maildb.DbQuery("SELECT ID,UID,BODY,MSGLEN,DELETED,CREATED,TYPE FROM messages WHERE DELETED=0 AND UID=%d" % userid)
    return map( lambda x: MysqlMessageEntry(x), table )   

def GetMessageByID(messageid):
    "Given a message ID, return the associated message object"
    try:
        return MysqlMessageEntry(maildb.DbQuery("SELECT ID,UID,BODY,MSGLEN,DELETED,CREATED,TYPE FROM messages WHERE ID=%d" % messageid)[0])
    except:
        return None
    
