#! /usr/bin/python

#
# ShicksUser.py
#                                                                      
# Copyright (C) 2001,2002 Gerson Kurz (not@p-nand-q.com)
# BSD-Licensed. This is free software, enjoy!

"""

This class defines Shicks! User management.

- A Shicks! User is a mailbox account that you can access via POP3
- A Shicks! User can have any number of email addresses associated with it
- A Shicks! User can have some settings associated with it (see below)
- A Shicks! User can be created, or destroyed; its email addresses can be created or destroyed; and its settings can be created or destroyed.
- The usermanagement can lookup a single user
- Each user must have a unique ID (expressed as a 32-bit integer)

"""

import maildb, traceback, rotor, binascii

class MysqlShicksUser:

    def __init__(self, userid, data = None):
        if data is not None:
            self.userid = data[0]
            self.name = data[1]
            self.password = rotor.newrotor("heinz",42).decrypt(binascii.a2b_hex(data[2]))
            self.checkspam = data[3]
            self.plaintext = data[4]            
        else:
            self.userid = userid
            result = maildb.DbQuery( "SELECT NAME,PASSWD,CHECKSPAM,PLAINTEXTONLY FROM users WHERE ID=%d" % self.userid )[0]
            self.name = result[0]
            self.password = rotor.newrotor("heinz",42).decrypt(binascii.a2b_hex(result[1]))
            self.checkspam = result[2]
            self.plaintext = result[3]

        # now, determine address list
        self.addresses = maildb.DbQuery("SELECT MAIL FROM addresses WHERE UID=%d" % self.userid)
        self.forwarders = maildb.DbQuery("SELECT a.UID,a.MAIL FROM forwardmail AS f, addresses AS a WHERE f.UID=%d and a.ID=f.AID" % self.userid)

    def getId(self):
        return self.userid

    def getPassword(self):
        return self.password

    def setPassword(self, password):
        self.password = password

    def getName(self):
        return self.name

    def setName(self, name):
        self.name = name

    def getPlaintextOnly(self):
        return self.plaintext

    def setPlaintextOnly(self, plaintext):
        self.plaintext = plaintext

    def getSpamCheck(self):
        return self.checkspam
        
    def setSpamCheck(self, checkspam):
        self.checkspam = checkspam

    def hasAddress(self, address):
        return address in self.addresses

    def getForwarders(self):
        return self.forwarders
        
    def saveChanges(self):
        enc_password = binascii.b2a_hex(rotor.newrotor("heinz",42).encrypt(self.password))
        maildb.DbCommit("UPDATE users SET NAME='%s',PASSWD='%s',CHECKSPAM=%d,PLAINTEXTONLY=%d WHERE ID=%d" % (self.name,enc_password,self.checkspam,self.plaintext,self.userid))

_userdict = {}

# ------------------ METHODS NEEDED FOR POP3Server.py / SMTPServer.py ------------------------------

def GetUserDict():
    "Returns a dictionary of all known users"
    try:
        rows = maildb.DbQuery( "SELECT ID,NAME,PASSWD,CHECKSPAM,PLAINTEXTONLY FROM users ORDER BY NAME" )
        _userdict = {}
        for row in rows:
            userid = row[0]
            _userdict[userid] = MysqlShicksUser(userid)
    except:
        pass
    return _userdict
    
def GetUserByName(name):
    "Returns a ShicksUser object for a given name or None if the user is not known"
    try:
        data = maildb.DbQuery("SELECT ID,NAME,PASSWD,CHECKSPAM,PLAINTEXTONLY FROM users WHERE NAME='%s'" % maildb.EscapeSqlString(name))
        if data:
            data = data[0]
            _userdict[data[0]] = MysqlShicksUser(data[0], data)
            return _userdict[data[0]]
    except:
        print "EXCEPTION in GetUserByName('%s')" % name
        traceback.print_exc()

def GetUserByAddress(address):
    "Returns a ShicksUser object for a given email address or None if the user is not known"
    try:
        data = maildb.DbQuery("SELECT a.UID,u.NAME,u.PASSWD,u.CHECKSPAM,u.PLAINTEXTONLY FROM addresses AS a, users AS u WHERE a.mail='%s' and u.ID=a.UID" % maildb.EscapeSqlString(address))
        if data:
            data = data[0]
            _userdict[data[0]] = MysqlShicksUser(data[0], data)
            return _userdict[data[0]]
    except:
        print "EXCEPTION in GetUserByAddress('%s')" % address
        traceback.print_exc()

# ------------------ METHODS NEEDED FOR ShicksAdmin.py ------------------------------

def GetUserByID(userid):
    "Returns a ShicksUser object for a given userid or None if the user is not known"
    try:
        _userdict[userid] = MysqlShicksUser(userid)
        return _userdict[userid]
    except:
        print "EXCEPTION in GetUserByID(%d)" % userid
        traceback.print_exc()

def DeleteUserByID(userid):
    "Deletes a user account, given its ID. Does NOT delete associated messages / addresses etc."
    try:
        maildb.DbCommit("DELETE FROM users WHERE ID=%d" % userid)
        del _userdict[userid]
    except:
        pass

def CreateNewUser(name,password,checkspam=1,plaintext=0):
    "Creates a new user account"
    password = binascii.b2a_hex(rotor.newrotor("heinz",42).encrypt(password))
    maildb.DbCommit("INSERT INTO users (NAME,PASSWD,CHECKSPAM, PLAINTEXTONLY) VALUES ('%s','%s',%d, %d)" % (name, password, checkspam, plaintext))
    
